@@ -15,6 +15,7 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
+	"k8s.io/utils/pointer"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	performancev2 "github.com/openshift-kni/performance-addon-operators/api/v2"
@@ -73,18 +74,23 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 		Expect(err).ToNot(HaveOccurred())
 		workerRTNodes, err = nodes.MatchingOptionalSelector(workerRTNodes)
 		Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("error looking for the optional selector: %v", err))
-		Expect(workerRTNodes).ToNot(BeEmpty())
+		Expect(workerRTNodes).ToNot(BeEmpty(), "cannot find RT enabled worker nodes")
 		profile, err = profiles.GetByNodeLabels(nodeLabel)
 		Expect(err).ToNot(HaveOccurred())
 		performanceMCP, err = mcps.GetByProfile(profile)
 		Expect(err).ToNot(HaveOccurred())
+
+		// Verify that worker and performance MCP have updated state equals to true
+		for _, mcpName := range []string{testutils.RoleWorker, performanceMCP} {
+			mcps.WaitForCondition(mcpName, machineconfigv1.MachineConfigPoolUpdated, corev1.ConditionTrue)
+		}
 	})
 
 	Context("Verify GloballyDisableIrqLoadBalancing Spec field", func() {
 		It("[test_id:36150] Verify that IRQ load balancing is enabled/disabled correctly", func() {
 			irqLoadBalancingDisabled := profile.Spec.GloballyDisableIrqLoadBalancing != nil && *profile.Spec.GloballyDisableIrqLoadBalancing
 
-			Expect(profile.Spec.CPU.Isolated).NotTo(BeNil())
+			Expect(profile.Spec.CPU.Isolated).NotTo(BeNil(), "expected isolated CPUs, found none")
 			isolatedCPUSet, err := cpuset.Parse(string(*profile.Spec.CPU.Isolated))
 			Expect(err).ToNot(HaveOccurred())
 
@@ -100,7 +106,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 					By(fmt.Sprintf("verifying worker node %q", node.Name))
 
 					bannedCPUs, err := nodes.BannedCPUs(node)
-					Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to extract the banned CPUs from node %s", node.Name))
+					Expect(err).ToNot(HaveOccurred(), "failed to extract the banned CPUs from node %s", node.Name)
 
 					if !bannedCPUs.Equals(expectedBannedCPUs) {
 						return fmt.Errorf("banned CPUs %v do not match the expected mask %v on node %s",
@@ -138,7 +144,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 			profile.Spec.GloballyDisableIrqLoadBalancing = &irqLoadBalancingDisabled
 
 			By("Updating the performance profile")
-			Expect(testclient.Client.Update(context.TODO(), profile)).ToNot(HaveOccurred())
+			profiles.UpdateWithRetry(profile)
 			defer func() { // return initial configuration
 				spec, err := json.Marshal(initialProfile.Spec)
 				Expect(err).ToNot(HaveOccurred())
@@ -154,6 +160,82 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 		})
 	})
 
+	Context("Verify hugepages count split on two NUMA nodes", func() {
+		hpSize2M := performancev2.HugePageSize("2M")
+
+		table.DescribeTable("Verify that profile parameters were updated", func(hpCntOnNuma0 int32, hpCntOnNuma1 int32) {
+			By("Verifying cluster configuration matches the requirement")
+			for _, node := range workerRTNodes {
+				numaInfo, err := nodes.GetNumaNodes(&node)
+				Expect(err).ToNot(HaveOccurred())
+				if len(numaInfo) < 2 {
+					Skip(fmt.Sprintf("This test need 2 NUMA nodes.The number of NUMA nodes on node %s < 2", node.Name))
+				}
+			}
+			//have total of 4 cpus so VMs can handle running the configuration
+			numaInfo, _ := nodes.GetNumaNodes(&workerRTNodes[0])
+			cpuSlice := numaInfo[0][0:4]
+			isolated := performancev2.CPUSet(fmt.Sprintf("%d-%d", cpuSlice[2], cpuSlice[3]))
+			reserved := performancev2.CPUSet(fmt.Sprintf("%d-%d", cpuSlice[0], cpuSlice[1]))
+
+			By("Modifying profile")
+			initialProfile = profile.DeepCopy()
+			profile.Spec.CPU = &performancev2.CPU{
+				BalanceIsolated: pointer.BoolPtr(false),
+				Reserved:        &reserved,
+				Isolated:        &isolated,
+			}
+			profile.Spec.HugePages = &performancev2.HugePages{
+				DefaultHugePagesSize: &hpSize2M,
+				Pages: []performancev2.HugePage{
+					{
+						Count: hpCntOnNuma0,
+						Size:  hpSize2M,
+						Node:  pointer.Int32Ptr(0),
+					},
+					{
+						Count: hpCntOnNuma1,
+						Size:  hpSize2M,
+						Node:  pointer.Int32Ptr(1),
+					},
+				},
+			}
+			profile.Spec.RealTimeKernel = &performancev2.RealTimeKernel{
+				Enabled: pointer.BoolPtr(true),
+			}
+
+			By("Verifying that mcp is ready for update")
+			mcps.WaitForCondition(performanceMCP, machineconfigv1.MachineConfigPoolUpdated, corev1.ConditionTrue)
+
+			By("Applying changes in performance profile and waiting until mcp will start updating")
+			profiles.UpdateWithRetry(profile)
+			mcps.WaitForCondition(performanceMCP, machineconfigv1.MachineConfigPoolUpdating, corev1.ConditionTrue)
+
+			By("Waiting when mcp finishes updates")
+			mcps.WaitForCondition(performanceMCP, machineconfigv1.MachineConfigPoolUpdated, corev1.ConditionTrue)
+
+			for _, node := range workerRTNodes {
+				for i := 0; i < 2; i++ {
+					nodeCmd := []string{"cat", hugepagesPathForNode(i, 2)}
+					result, err := nodes.ExecCommandOnNode(nodeCmd, &node)
+					Expect(err).ToNot(HaveOccurred())
+
+					t, err := strconv.Atoi(result)
+					Expect(err).ToNot(HaveOccurred())
+
+					if i == 0 {
+						Expect(int32(t)).To(Equal(hpCntOnNuma0))
+					} else {
+						Expect(int32(t)).To(Equal(hpCntOnNuma1))
+					}
+				}
+			}
+		},
+			table.Entry("[test_id:45023] verify uneven split of hugepages between 2 numa nodes", int32(2), int32(1)),
+			table.Entry("[test_id:45024] verify even split between 2 numa nodes", int32(1), int32(1)),
+		)
+	})
+
 	Context("Verify that all performance profile parameters can be updated", func() {
 		var removedKernelArgs string
 
@@ -162,7 +244,6 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 		isolated := performancev2.CPUSet("1-2")
 		reserved := performancev2.CPUSet("0,3")
 		policy := "best-effort"
-		f := false
 
 		// Modify profile and verify that MCO successfully updated the node
 		testutils.BeforeAll(func() {
@@ -183,7 +264,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 				},
 			}
 			profile.Spec.CPU = &performancev2.CPU{
-				BalanceIsolated: &f,
+				BalanceIsolated: pointer.BoolPtr(false),
 				Reserved:        &reserved,
 				Isolated:        &isolated,
 			}
@@ -191,7 +272,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 				TopologyPolicy: &policy,
 			}
 			profile.Spec.RealTimeKernel = &performancev2.RealTimeKernel{
-				Enabled: &f,
+				Enabled: pointer.BoolPtr(false),
 			}
 
 			if profile.Spec.AdditionalKernelArgs == nil {
@@ -206,7 +287,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 			mcps.WaitForCondition(performanceMCP, machineconfigv1.MachineConfigPoolUpdated, corev1.ConditionTrue)
 
 			By("Applying changes in performance profile and waiting until mcp will start updating")
-			Expect(testclient.Client.Update(context.TODO(), profile)).ToNot(HaveOccurred())
+			profiles.UpdateWithRetry(profile)
 			mcps.WaitForCondition(performanceMCP, machineconfigv1.MachineConfigPoolUpdating, corev1.ConditionTrue)
 
 			By("Waiting when mcp finishes updates")
@@ -253,14 +334,14 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 		It("[test_id:28612]Verify that Kernel arguments can me updated (added, removed) thru performance profile", func() {
 			for _, node := range workerRTNodes {
 				cmdline, err := nodes.ExecCommandOnNode(chkCmdLine, &node)
-				Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to execute %s", chkCmdLine))
+				Expect(err).ToNot(HaveOccurred(), "failed to execute %s", chkCmdLine)
 
 				// Verifying that new argument was added
 				Expect(cmdline).To(ContainSubstring("new-argument=test"))
 
 				// Verifying that one of old arguments was removed
 				if removedKernelArgs != "" {
-					Expect(cmdline).NotTo(ContainSubstring(removedKernelArgs), fmt.Sprintf("%s should be removed from /proc/cmdline", removedKernelArgs))
+					Expect(cmdline).NotTo(ContainSubstring(removedKernelArgs), "%s should be removed from /proc/cmdline", removedKernelArgs)
 				}
 			}
 		})
@@ -274,9 +355,9 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 
 			By("Applying changes in performance profile")
 			profile.Spec.RealTimeKernel = nil
-			Expect(testclient.Client.Update(context.TODO(), profile)).ToNot(HaveOccurred())
+			profiles.UpdateWithRetry(profile)
 
-			Expect(profile.Spec.RealTimeKernel).To(BeNil())
+			Expect(profile.Spec.RealTimeKernel).To(BeNil(), "real time kernel setting expected in profile spec but missing")
 			By("Checking that the updating MCP status will consistently stay false")
 			Consistently(func() corev1.ConditionStatus {
 				return mcps.GetConditionStatus(performanceMCP, conditionUpdating)
@@ -293,7 +374,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 			spec, err := json.Marshal(initialProfile.Spec)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(testclient.Client.Patch(context.TODO(), profile,
-				client.ConstantPatch(
+				client.RawPatch(
 					types.JSONPatchType,
 					[]byte(fmt.Sprintf(`[{ "op": "replace", "path": "/spec", "value": %s }]`, spec)),
 				),
@@ -337,18 +418,18 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 
 			By("Updating Node Selector performance profile")
 			profile.Spec.NodeSelector = newNodeSelector
-			Expect(testclient.Client.Update(context.TODO(), profile)).ToNot(HaveOccurred())
+			profiles.UpdateWithRetry(profile)
 			mcps.WaitForCondition(newRole, machineconfigv1.MachineConfigPoolUpdating, corev1.ConditionTrue)
 
 			By("Waiting when MCP finishes updates and verifying new node has updated configuration")
 			mcps.WaitForCondition(newRole, machineconfigv1.MachineConfigPoolUpdated, corev1.ConditionTrue)
 
 			kblcfg, err := nodes.ExecCommandOnNode(chkKubeletConfig, newCnfNode)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to execute %s", chkKubeletConfig))
+			Expect(err).ToNot(HaveOccurred(), "failed to execute %s", chkKubeletConfig)
 			Expect(kblcfg).To(ContainSubstring("topologyManagerPolicy"))
 
 			cmdline, err := nodes.ExecCommandOnNode(chkCmdLine, newCnfNode)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to execute %s", chkCmdLine))
+			Expect(err).ToNot(HaveOccurred(), "failed to execute %s", chkCmdLine)
 			Expect(cmdline).To(ContainSubstring("tuned.non_isolcpus"))
 		})
 
@@ -360,7 +441,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 			label, err := json.Marshal(newCnfNode.Labels)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(testclient.Client.Patch(context.TODO(), newCnfNode,
-				client.ConstantPatch(
+				client.RawPatch(
 					types.JSONPatchType,
 					[]byte(fmt.Sprintf(`[{ "op": "replace", "path": "/metadata/labels", "value": %s }]`, label)),
 				),
@@ -382,19 +463,19 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 			Expect(err).To(HaveOccurred())
 
 			cmdline, err := nodes.ExecCommandOnNode(chkCmdLine, newCnfNode)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to execute %s", chkCmdLine))
+			Expect(err).ToNot(HaveOccurred(), "failed to execute %s", chkCmdLine)
 			Expect(cmdline).NotTo(ContainSubstring("tuned.non_isolcpus"))
 
 			kblcfg, err := nodes.ExecCommandOnNode(chkKubeletConfig, newCnfNode)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to execute %s", chkKubeletConfig))
+			Expect(err).ToNot(HaveOccurred(), "failed to execute %s", chkKubeletConfig)
 			Expect(kblcfg).NotTo(ContainSubstring("reservedSystemCPUs"))
 
 			Expect(profile.Spec.CPU.Reserved).NotTo(BeNil())
 			reservedCPU := string(*profile.Spec.CPU.Reserved)
 			cpuMask, err := components.CPUListToHexMask(reservedCPU)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to list in Hex %s", reservedCPU))
+			Expect(err).ToNot(HaveOccurred(), "failed to list in Hex %s", reservedCPU)
 			irqBal, err := nodes.ExecCommandOnNode(chkIrqbalance, newCnfNode)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("failed to execute %s", chkIrqbalance))
+			Expect(err).ToNot(HaveOccurred(), "failed to execute %s", chkIrqbalance)
 			Expect(irqBal).NotTo(ContainSubstring(cpuMask))
 		})
 
@@ -405,7 +486,7 @@ var _ = Describe("[rfe_id:28761][performance] Updating parameters in performance
 			}
 			nodeSelector := strings.Join(selectorLabels, ",")
 			Expect(testclient.Client.Patch(context.TODO(), profile,
-				client.ConstantPatch(
+				client.RawPatch(
 					types.JSONPatchType,
 					[]byte(fmt.Sprintf(`[{ "op": "replace", "path": "/spec/nodeSelector", "value": {%s} }]`, nodeSelector)),
 				),
